/*
** Copyright (C) 2004-2008 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
**
** Use of the SILK system and related source code is subject to the terms
** of the following licenses:
**
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
**
** NO WARRANTY
**
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE,
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT,
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES.
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE
** DELIVERABLES UNDER THIS LICENSE.
**
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie
** Mellon University, its trustees, officers, employees, and agents from
** all claims or demands made against them (and any related losses,
** expenses, or attorney's fees) arising out of, or relating to Licensee's
** and/or its sub licensees' negligent use or willful misuse of or
** negligent conduct or willful misconduct regarding the Software,
** facilities, or other rights or assistance granted by Carnegie Mellon
** University under this License, including, but not limited to, any
** claims of product liability, personal injury, death, damage to
** property, or violation of any laws or regulations.
**
** Carnegie Mellon University Software Engineering Institute authored
** documents are sponsored by the U.S. Department of Defense under
** Contract F19628-00-C-0003. Carnegie Mellon University retains
** copyrights in all material produced under this contract. The U.S.
** Government retains a non-exclusive, royalty-free license to publish or
** reproduce these documents, or allow others to do so, for U.S.
** Government purposes only pursuant to the copyright license under the
** contract clause at 252.227.7013.
**
** @OPENSOURCE_HEADER_END@
*/
#ifndef _STREAM_CACHE_H
#define _STREAM_CACHE_H

#include "silk.h"

RCSIDENTVAR(rcsID_STREAM_CACHE_H, "$SiLK: stream-cache.h 10004 2008-01-04 23:08:59Z mthomas $");

/*
**  stream-cache.h
**
**  A simple interface for maintaining a list of open file handles so
**  we can avoid a lot of open/close cycles.  File handles are indexed
**  by the timestamp of the file, the sensor_id, and the flowtype
**  (class/type) of the data they contain.
*/

#include "rwpack.h"


#define STREAM_CACHE_MINIMUM_SIZE 2
/*
 *    Smallest maximum cache size.  Code that handles removing items
 *    from end of list assumes at least two entries in the list.
 */


struct stream_cache_st;
typedef struct stream_cache_st stream_cache_t;


/*
 *  cache = skStreamCacheCreate(max_size);
 *
 *    Create a stream_cache capable of keeping 'max_size' files open.
 *    Returns NULL if memory cannot be allocated.
 */
stream_cache_t *skStreamCacheCreate(int max_size);

/*
 *  skStreamCacheDestroy(cache);
 *
 *    Close all streams and free all memory associated with the
 *    streams.  Free the memory associated with the cache.  The cache
 *    pointer is invalid after a call to these function.
 */
void skStreamCacheDestroy(stream_cache_t *c);

/*
 *  skStreamCacheFlush(cache);
 *
 *    Flush all the streams in the cache, and log the number of
 *    records processed since the most recent flush or open.
 */
void skStreamCacheFlush(stream_cache_t *c);


/*
 *  skStreamCacheCloseAll(cache);
 *
 *    Close all the the streams in the cache and remove them from the
 *    cache.  For each file, log the number of records processed since
 *    the most recent flush or open.
 */
void skStreamCacheCloseAll(stream_cache_t *cache);


/*
 *  skStreamCacheAdd(cache, stream, timestamp, sensor, flowtype);
 *
 *    Add 'stream' to the stream cache 'cache'.  Use the 'timestamp',
 *    'sensor', and 'flowtype' as the key..  After this call, the
 *    cache will own stream memory and will free it when the stream
 *    falls off the end of the cache, or when the cache is destroyed.
 */
int skStreamCacheAdd(
    stream_cache_t  *c,
    rwIOStruct_t    *s,
    sktime_t         time_stamp,
    sensorID_t       sensor_id,
    flowtypeID_t     flowtype_id);

/*
 *  stream = skStreamCacheLookup(cache, time_stamp, sensor_id, flowtype_id);
 *
 *    Return the stream for the specified timestamp, sensor, and
 *    flowtype.  Return NULL if no stream for the specified values is
 *    found.
 */
rwIOStruct_t *skStreamCacheLookup(
    stream_cache_t  *c,
    sktime_t        time_stamp,
    sensorID_t      sensor_id,
    flowtypeID_t    flowtype_id);


#endif /* _STREAM_CACHE_H */

/*
** Local Variables:
** mode:c
** indent-tabs-mode:nil
** c-basic-offset:4
** End:
*/
